;;;  cltio-high.lisp --- high level interface

;;;  Copyright (C) 2004 by Walter C. Pelissero

;;;  Author: Walter C. Pelissero <walter@pelissero.de>
;;;  Project: Common Lisp Terminal I/O
;;;  $Id: F-45DC2B40C6721FA128E260DEDE238085.lisp,v 1.1.1.1 2005/01/18 22:29:11 wcp Exp $

;;; This library is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Lesser General Public License
;;; as published by the Free Software Foundation; either version 2.1
;;; of the License, or (at your option) any later version.
;;; This library is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Lesser General Public License for more details.
;;; You should have received a copy of the GNU Lesser General Public
;;; License along with this library; if not, write to the Free
;;; Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
;;; 02111-1307 USA


#+cmu (ext:file-comment "$Id: F-45DC2B40C6721FA128E260DEDE238085.lisp,v 1.1.1.1 2005/01/18 22:29:11 wcp Exp $")

(in-package :cltio)

#-(or cmu sbcl)
(error "This code hasn't been tested on your Lisp system.")

(defun file-descriptor (object)
  #+cmu
  (if (system:fd-stream-p object)
      (system:fd-stream-fd object)
      object)
  #+sbcl
  (if (sb-sys:fd-stream-p object)
      (sb-sys:fd-stream-fd object)
      object))

(defun get-termios (fd)
  (let ((tc (allocate-foreign-object 'termios)))
    (tcgetattr (file-descriptor fd) tc)
    tc))

(defun set-termios (fd termios &optional (when-list '(:now)))
  (tcsetattr (file-descriptor fd)
	     (reduce #'logior (mapcar #'(lambda (action)
					  (cadr (assoc action `((:now ,tcsanow)
								(:drain ,tcsadrain)
								(:flush ,tcsaflush)
								(:soft ,tcsasoft))))) when-list))
	     termios))

(defun stty (fd &key parity cs speed raw local)
  (let ((tc (get-termios (file-descriptor fd))))
    (case parity
      (:even
       (setf (get-slot-value tc 'termios 'c_cflag)
	     (logandc2 (logior (get-slot-value tc 'termios 'c_cflag) parenb)
			parodd)))
      (:odd
       (setf (get-slot-value tc 'termios 'c_cflag)
	     (logior (get-slot-value tc 'termios 'c_cflag) parenb parodd)))
      (:none
       (setf (get-slot-value tc 'termios 'c_cflag)
	     (logandc2 (get-slot-value tc 'termios 'c_cflag) (logior parenb parodd)))))
    (when cs
      (setf (get-slot-value tc 'termios 'c_cflag)
	    (logior (logandc2 (get-slot-value tc 'termios 'c_cflag) csize)
		    (cond ((= 8 cs) cs8)
			  ((= 7 cs) cs7)
			  ((= 6 cs) cs6)
			  (t 0)))))
    (when speed
      (cfsetspeed tc speed))
    (when raw
      (cfmakeraw tc))
    (cond ((eq local :no)
	   (setf (get-slot-value tc 'termios 'c_cflag)
		 (logandc2 (get-slot-value tc 'termios 'c_cflag) clocal)))
	  (local
	   (setf (get-slot-value tc 'termios 'c_cflag)
		 (logior (get-slot-value tc 'termios 'c_cflag) clocal))))
    (set-termios (file-descriptor fd) tc)))

(defmacro with-stty ((fd &rest stty-args) &body body)
  (let ((old-tc (gensym)))
    `(let ((,old-tc (get-termios ,fd)))
       (stty ,fd ,@stty-args)
       (unwind-protect (progn ,@body)
	 (set-termios ,fd ,old-tc)))))

(export '(with-stty stty get-termios set-termios file-descriptor))
